<?php
// $Id: fixcore.admin.inc,v 1.3 2010/03/06 03:51:45 salvis Exp $

/**
 * @file
 * This module collects various fixes for Drupal core issues.
 *
 * It would be nice to get them committed to core at some point,
 * but in the meantime this can ease the pain.
 */

/**
 * Settings form at admin/settings/fixcore.
 */
function fixcore_settings_form(&$form_state) {
  global $user;

  $tr = 't';
  $form['intro'][] = array(
    '#type'          => 'item',
    '#value'         => t('This module collects various fixes to core issues that have not been implemented yet. By default, all of these fixes are disabled, and you can enable them one by one. <b>You do this at your own risk!</b> Be sure to test each fix before implementing it on a live site.'),
  );
  $form['intro'][] = array(
    '#type'          => 'item',
    '#value'         => t("Whenever you update Drupal, you must check whether the enabled fixes still apply. At some point, some of the issues may get fixed in core, and the corresponding fix in this module may then start to cause problems, if it's still enabled."),
  );

  $form['admin'][] = array(
    '#type'          => 'item',
    '#value'         => '<h3>'. t('Administrator issues') .'</h3>',
  );
  $form['admin']['access_time'] = array(
    '#type'          => 'fieldset',
    '#title'         => t("Avoid clobbering the user's access time on the !link page through administrative action (!Issue_link)", array('!link' => '<a href="'. url('admin/user/user') .'">admin/user/user</a>', '!Issue_link' => '<a href="'. url('http://drupal.org/node/171117', array('absolute' => TRUE, 'external' => TRUE)) .'">'. t('Issue link') .'</a>')),
    '#description'   => t("This bug was introduced in <a href='!url'>Drupal 6.00 BETA3</a> (!cvs_link) and it seems impossible to get fixed. It's the reason for starting this module.", array('!url' => url('http://drupal.org/node/171117#comment-622067', array('absolute' => TRUE, 'external' => TRUE)), '!cvs_link' => '<a href="'. url('http://drupalcode.org/viewvc/drupal/drupal/modules/user/user.module?r1=1.861&r2=1.862', array('absolute' => TRUE, 'external' => TRUE)) .'">cvs</a>')),
  );
  $form['admin']['access_time']['fixcore_no_ghost_users'] = array(
    '#type'          => 'checkbox',
    '#title'         => t("Don't fake access, preserve '%never'", array('%never' => $tr('never'))),
    '#default_value' => variable_get('fixcore_no_ghost_users', FALSE),
    '#description'   => t("If you create a new user, core immediately sets their %Last_access to the same time as their %Member_for value. Correct behavior would be to show '%never' until the user has actually accessed the site, so that you can easily tell whether a user has activated their account or not. Also, users should not appear in the %Who_s_new block if they've never accessed your site. <br /><br /> If you want to fix existing users and find out who has never accessed your site, run the following query: !SQL The downside of enabling this fix is the following: the user pages of users with %condition are inaccessible to other users, and if you have a buggy module or view that displays a list of user records and fails to filter out the %condition records, then you may get links to 'access denied' (403) pages; this needs to be fixed in that module or view!", array('%Last_access' => $tr('Last access'), '%Member_for' => $tr('Member for'), '%never' => $tr('never'), '!SQL' => '<br /><pre>UPDATE {users} SET access = 0 WHERE access = created;</pre>', '%Who_s_new' => $tr("Who's new"), '%condition' => 'access=0')),
  );
  $form['admin']['access_time']['fixcore_preserve_access_time'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Preserve access time when editing an existing user'),
    '#default_value' => variable_get('fixcore_preserve_access_time', FALSE),
    '#description'   => t('When an administrator edits a user, e.g. to block them, core sets their %Last_access to the current time. Correct behavior would be to leave %Last_access unchanged.', array('%Last_access' => $tr('Last access'))),
  );
  $form['admin']['fixcore_avoid_msoffice_404'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Avoid MSOffice Server Extensions %page_not_found errors (!Read_more)', array('%page_not_found' => $tr('page not found'), '!Read_more' => '<a href="'. url('http://www.msoffice-cltreq-asp.info/', array('absolute' => TRUE, 'external' => TRUE)) .'">'. $tr('Read more') .'</a>')),
    '#default_value' => variable_get('fixcore_avoid_msoffice_404', FALSE),
    '#description'   => t("If you're tired of seeing %page_not_found errors for %file1 and %file2, then turn this on.", array('%page_not_found' => $tr('page not found'), '%file1' => 'MSOffice/cltreq.asp', '%file2' => '_vti_bin/owssvr.dll')),
  );

  $form['end_user'][] = array(
    '#type'          => 'item',
    '#value'         => '<h3>'. t('End user issues') .'</h3>',
  );
  $form['end_user']['fixcore_password'] = array(
    '#type'          => 'fieldset',
    '#title'         => t('Newbie users (not) changing their password (!Issue_link)', array('!Issue_link' => '<a href="'. url('http://drupal.org/node/601146', array('absolute' => TRUE, 'external' => TRUE)) .'">'. t('Issue link') .'</a>')),
    '#description'   => t('This is a severe usability and maintenance issue: experience shows that newbie users who log in via a log-in link often forget to click the !Save button after changing their password. This causes endless frustration to your users and to you, because they always fail to log in.', array('!Save' => '['. $tr('Save') .']')),
  );
  $contribute_translation = t("If your site is non-English, you should check whether the hint appears properly translated. If it doesn't, then please contribute the correct translation back to the !fixcore project.", array('!fixcore' => '<a href="'. url('http://drupal.org/project/fixcore', array('absolute' => TRUE, 'external' => TRUE)) .'">'. 'fixcore</a>'));
  $form['end_user']['fixcore_password']['fixcore_password_save_hint_static'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Provide a static %Save hint on the !link page', array('%Save' => $tr('Save'), '!link' => '<a href="'. url('user/'. $user->uid .'/edit') .'">user/'. $user->uid .'/edit</a>')),
    '#default_value' => variable_get('fixcore_password_save_hint_static', FALSE),
    '#description'   => t('This adds a reminder to the description below the password change widget.') .'<br />'. $contribute_translation,
  );
  $form['end_user']['fixcore_password']['fixcore_password_save_hint_js'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Provide a JavaScript %Save hint on the !link page', array('%Save' => $tr('Save'), '!link' => '<a href="'. url('user/'. $user->uid .'/edit') .'">user/'. $user->uid .'/edit</a>')),
    '#default_value' => variable_get('fixcore_password_save_hint_js', FALSE),
    '#description'   => t('This adds a dynamic reminder to the password change widget (and suppresses the static reminder, if JavaScript is enabled).') .'<br />'. $contribute_translation,
  );
  $form['end_user']['fixcore_password']['fixcore_log_password_change'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Write a !watchdog entry when the user password is updated', array('!watchdog' => $tr('watchdog'))),
    '#default_value' => variable_get('fixcore_log_password_change', FALSE),
    '#description'   => t('Knowing whether they did change their password or not will help you to get them over that critical hurdle.', array('!Save' => '['. $tr('Save') .']')),
  );
  $form['end_user']['fixcore_collapse_revision_info'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Collapse the %Revision_information fieldset for new nodes', array('%Revision_information' => $tr('Revision information'))),
    '#default_value' => variable_get('fixcore_collapse_revision_info', FALSE),
    '#description'   => t('This is a usability issue: if you turn on %Create_new_revision for some content type and you have regular users creating nodes of that type, then these users are often confused by the open %Revision_information fieldset. This is rarely needed for new nodes.', array('%Create_new_revision' => $tr('Create new revision'), '%Revision_information' => $tr('Revision information'))),
  );
  $form['end_user']['fixcore_login_to_frontpage'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Redirect logins to the front page'),
    '#default_value' => variable_get('fixcore_login_to_frontpage', FALSE),
    '#description'   => t('This is a usability issue: The Drupal default is to redirect to the dull and ugly user page; turn this on to redirect to the (hopefully now enhanced) front page. Logins through the Login block are not affected.'),
  );
  if (module_exists('contact')) {
    $form['end_user']['fixcore_contact_select'] = array(
      '#type'          => 'checkbox',
      '#title'         => t('Allow preselecting the contact category'),
      '#default_value' => variable_get('fixcore_contact_select', FALSE),
      '#description'   => t('This is a usability issue: the <a href="@link">site-wide contact form</a> does not allow preselecting a category, and thus you cannot create category-specific links. Turn this on and add the index of the desired category to the path, such as !path.', array('@link' => url('contact'), '!path' => '<a href="'. url('contact/1') .'">contact/1</a>')),
    );
  }

  $form['display'][] = array(
    '#type'          => 'item',
    '#value'         => '<h3>'. t('Display issues') .'</h3>',
    '#description'   => t('No I-prefer-it-this-way tweaks, only fixes for obvious bugs that interfere with intended use!'),
  );
  $form['display']['fixcore_multiselect_autosize'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Make multiselect definition lists auto-sized'),
    '#default_value' => variable_get('fixcore_multiselect_autosize', FALSE),
    '#description'   => t("Drupal core's system.css gives only a hard-coded 8em width to multiselect definition lists such as the one on !link. This is not enough for some translations, e.g. German, resulting in a garbled display.", array('!link' => '<a href="'. url('admin/user/user') .'">admin/user/user</a>')),
  );
  $form['display']['fixcore_node_unpublished'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Make the background of unpublished nodes a darker shade of red'),
    '#default_value' => variable_get('fixcore_node_unpublished', FALSE),
    '#description'   => t("Users with the %administer_nodes permission have access to unpublished nodes, and Drupal core's node.css displays them with a reddish background. If it looks white to you, then turn this on to make it redder.", array('%administer_nodes' => 'administer nodes')),
  );

  $form['development'][] = array(
    '#type'          => 'item',
    '#value'         => '<h3>'. t('Development issues') .'</h3>',
  );
  $form['development']['fixcore_debug_access_denied'] = array(
    '#type'          => 'checkbox',
    '#title'         => t("Provide a debug_backtrace() when user 1 gets an 'access denied' error (!Issue_link)", array('!Issue_link' => '<a href="'. url('http://drupal.org/node/610568', array('absolute' => TRUE, 'external' => TRUE)) .'">'. t('Issue link') .'</a>')),
    '#default_value' => variable_get('fixcore_debug_access_denied', FALSE),
    '#description'   => t('This should never happen, but when it does, you need to find out where it occurs. Install the !Devel module to see the result.', array('!Devel' => '<a href="'. url('http://drupal.org/project/devel', array('absolute' => TRUE, 'external' => TRUE)) .'">Devel</a>')),
  );
  if (module_exists('forum')) {
    $form['development']['fixcore_node_tid'] = array();
    if (VERSION >= '6.16') {
      $form['development']['fixcore_node_tid'] += array(
        '#disabled'      => TRUE,
        '#default_value' => FALSE,
      );
    }
    $form['development']['fixcore_node_tid'] += array(
      '#type'          => 'checkbox',
      '#title'         => t('Make sure !node_tid is the forum !tid (!Issue_link)', array('!node_tid' => '$node->tid', '!tid' => 'tid', '!Issue_link' => '<a href="'. url('http://drupal.org/node/422218', array('absolute' => TRUE, 'external' => TRUE)) .'">'. t('Issue link') .'</a>')),
      '#default_value' => variable_get('fixcore_node_tid', FALSE),
      '#description'   => t('This is a technical issue: if you have forum nodes with additional terms, !node_tid (which is supposed to be the forum id) can be the id of one of the other terms; core uses the lowest tid. This can confuse other modules.', array('!node_tid' => '$node->tid', '!tid' => 'tid')),
    );
    if (VERSION >= '6.16') {
      $form['development']['fixcore_node_tid']['#title'] = '<del>'. $form['development']['fixcore_node_tid']['#title'] .'</del> &nbsp; FIXED in 6.16!';
    }
  }

  $form = system_settings_form($form);  
  $form['#submit'][] = '_fixcore_settings_form_submit';
  return $form;
}

function _fixcore_settings_form_submit($form, &$form_state) {
  drupal_flush_all_caches();
}
